import re
import os
import sys
import subprocess
import textwrap
import decimal
from . import constants
from . import utils
from . import argparse_utils


def load_command(args):
    patterns = []
    for name in args.pool_names:
        context = vars(args)
        context['pool_name'] = name
        context['prefix'] %= context
        context['postfix'] %= context
        patterns.append(constants.IMAGE_NAME_RE % context)

    pattern = re.compile('(%s)' % '|'.join(patterns))
    full_path = os.path.abspath(os.path.expanduser(args.image_directory))
    for path, dirs, files in os.walk(full_path):
        for file_ in files:
            match = pattern.match(file_)
            if not match:
                continue
            try:
                context['name'] = os.path.join(path, file_)
                utils.execute( context, constants.ZPOOL_ATTACH_IMAGE_COMMAND)
            except subprocess.CalledProcessError:
                print 'Unable to attach image'
                sys.exit(1)

    for name in args.pool_names:
        context = vars(args)
        context['pool_name'] = name

        try:
            utils.execute( context, constants.ZPOOL_IMPORT_COMMAND)
        except subprocess.CalledProcessError:
            print 'Unable to import zpool'
            sys.exit(1)


def get_parser(subparsers):
    load = subparsers.add_parser('load', help='load (import) the zpools')
    load.add_argument('pool_names', help='The name of the pool to import',
                       nargs='+')
    load.add_argument(
        '-d', '--image-directory', default='~/zfs/',
        help='File name pattern to store the images (default: %(default)s)')
    load.add_argument(
        '-p', '--prefix', default='%(pool_name)s_',
        help='File name prefix for the images (default: %(default)s)')
    load.add_argument(
        '--postfix', default='',
        help='File name postfix for the images (default: %(default)s)')
    load.add_argument(
        '-n', '--no-op', '--dry-run', action='store_true',
        help='Show what will be done but dont execute')
    load.set_defaults(func=load_command)






ZPOOL_TYPES = dict(
    raidz=lambda n: n - 1,
    raidz1=lambda n: n - 1,
    raidz2=lambda n: n - 2,
    raidz3=lambda n: n - 3,
    mirror=lambda n: 1,
)

ZPOOL_CREATE_MESSAGE = '''
Creating %(count)s images named %(name)s of size %(image_size)fGiB
for an effective size of %(effective_size)fGiB. The disk space used will be
%(physical_size)fGiB.
'''.strip()

ZPOOL_CREATE_IMAGE_COMMAND = '''
hdiutil create
    -size %(image_size)sg
    -volname %(name)s
    -type SPARSE
    -layout NONE
    %(extra_args)s
    %(name)s
'''
ZPOOL_ATTACH_IMAGE_COMMAND = '''
hdiutil attach
    -nomount
    %(extra_args)s
    %(name)s
'''
ZPOOL_CREATE_COMMAND = '''
sudo zpool create
    %(extra_args)s
    -f
    -m %(mountpoint)s
    %(pool_name)s
    %(type)s
    %(devices)s
'''
ZPOOL_IMPORT_COMMAND = '''
sudo zpool import %(pool_name)s
'''
IMAGE_NAME = '%(prefix)s%(i)02d%(postfix)s'
IMAGE_NAME_RE = '(%(prefix)s)(\d{2})(%(postfix)s).sparseimage'




__package_name__ = 'zfs-utils-osx'
__version__ = '1.1.0'
__author__ = 'Rick van Hattem'
__author_email__ = 'Wolph@Wol.ph'
__description__ = '''
A simple script to create and manage virtual ZFS images on OS X without
requiring repartitioning
'''
__url__ = 'https://github.com/WoLpH/zfs-utils-osx/'




import subprocess


def execute(context, command, *args):
    command = ' '.join(command.split())
    context['extra_args'] = ' '.join(args)
    command %= context
    print '#', command
    if not context['no_op']:
        return subprocess.check_output(command, shell=True)


def coalesce(*arg):
    return reduce(lambda x, y: x if x is not None else y, arg)




#!/usr/bin/env python
import argparse
import zpool
import load


def get_parser():
    parser = argparse.ArgumentParser(description=__doc__)
    subparsers = parser.add_subparsers(
        title='Subcommands',
        description='Please specify one of the following subcommands')
    zpool.get_parser(subparsers)
    load.get_parser(subparsers)

    return parser


def main():
    parser = get_parser()
    args = parser.parse_args()
    args.func(args)


if __name__ == '__main__':
    main()




import sys
import subprocess
import textwrap
import decimal
from . import constants
from . import utils
from . import argparse_utils


def zpool_command(args):
    context = vars(args)
    effective_image_count = constants.ZPOOL_TYPES[args.type](args.count)
    context['image_size'] = args.size / effective_image_count
    context['physical_size'] = context['image_size'] * args.count
    context['effective_size'] = context['image_size'] * effective_image_count
    context['prefix'] %= context
    context['postfix'] %= context
    context['i'] = 0
    context['name'] = constants.IMAGE_NAME % context
    context['extra_args'] = ''
    print textwrap.fill(constants.ZPOOL_CREATE_MESSAGE % context)

    devices = []
    for i in range(args.count):
        context['i'] = i
        context['name'] = constants.IMAGE_NAME % context
        try:
            if args.overwrite:
                arg = '-ov'
            else:
                arg = ''

            utils.execute(context, constants.ZPOOL_CREATE_IMAGE_COMMAND, arg)
        except subprocess.CalledProcessError:
            print 'Unable to create a new image'
            sys.exit(1)

        try:
            context['name'] += '.sparseimage'
            device = utils.execute(context,
                                   constants.ZPOOL_ATTACH_IMAGE_COMMAND)
            if device:
                devices.append(device.strip())
        except subprocess.CalledProcessError:
            print 'Unable to attach image'
            sys.exit(1)

    if devices:
        context['devices'] = ' '.join(devices)
        context['mountpoint'] %= context
        utils.execute(context, constants.ZPOOL_CREATE_COMMAND)


def get_parser(subparsers):
    zpool = subparsers.add_parser('zpool', help='zpool creation')
    zpool.add_argument(
        '-c', '--count', default=3,
        type=lambda s: argparse_utils.greater_than(s, int, 1),
        help='The amount of images to use (default: %(default)s)')
    zpool.add_argument(
        '-s', '--size', default=10,
        type=lambda s: argparse_utils.greater_than(s, decimal.Decimal, 0),
        help='The usable size of the zpool in GiB (default: %(default)sGiB)')
    zpool.add_argument(
        '-t', '--type', choices=constants.ZPOOL_TYPES, default='raidz',
        help='The zpool type to use (default: %(default)s)')
    zpool.add_argument(
        '-n', '--no-op', '--dry-run', action='store_true',
        help='Show what will be done but dont execute')
    zpool.add_argument(
        '-m', '--mountpoint', default='~/%(pool_name)s',
        help='Where should the disk be mounted (default: %(default)s')
    zpool.add_argument(
        '-o', '--overwrite', action='store_true',
        help='Overwrite old images if they exist')
    zpool.add_argument('pool_name', help='The name of the pool to create')
    zpool.add_argument(
        '-p', '--prefix', default='%(pool_name)s_',
        help='File name prefix for the images (default: %(default)s)')
    zpool.add_argument(
        '--postfix', default='',
        help='File name postfix for the images (default: %(default)s)')
    zpool.set_defaults(func=zpool_command)




import argparse


def greater_than(string, cast, minimum):
    value = cast(string)
    if value > minimum:
        return value
    raise argparse.ArgumentTypeError('%r needs to be greater than %r' %
                                     (string, minimum))

